@@ -27,6 +27,16 @@
 
 #include "gststatistics.h"
 
+static GstStaticPadTemplate sinktemplate = GST_STATIC_PAD_TEMPLATE ("sink",
+    GST_PAD_SINK,
+    GST_PAD_ALWAYS,
+    GST_STATIC_CAPS_ANY);
+
+static GstStaticPadTemplate srctemplate = GST_STATIC_PAD_TEMPLATE ("src",
+    GST_PAD_SRC,
+    GST_PAD_ALWAYS,
+    GST_STATIC_CAPS_ANY);
+
 GST_DEBUG_CATEGORY_STATIC (gst_statistics_debug);
 #define GST_CAT_DEFAULT gst_statistics_debug
 
@@ -84,6 +94,10 @@ gst_statistics_base_init (gpointer g_class)
 {
   GstElementClass *gstelement_class = GST_ELEMENT_CLASS (g_class);
 
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&srctemplate));
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&sinktemplate));
   gst_element_class_set_details (gstelement_class, &gst_statistics_details);
 }
 
@@ -155,12 +169,16 @@ gst_statistics_class_init (GstStatisticsClass * klass)
 static void
 gst_statistics_init (GstStatistics * statistics)
 {
-  statistics->sinkpad = gst_pad_new ("sink", GST_PAD_SINK);
+  statistics->sinkpad =
+      gst_pad_new_from_template (gst_static_pad_template_get (&sinktemplate),
+      "sink");
   gst_element_add_pad (GST_ELEMENT (statistics), statistics->sinkpad);
   gst_pad_set_chain_function (statistics->sinkpad,
       GST_DEBUG_FUNCPTR (gst_statistics_chain));
 
-  statistics->srcpad = gst_pad_new ("src", GST_PAD_SRC);
+  statistics->srcpad =
+      gst_pad_new_from_template (gst_static_pad_template_get (&srctemplate),
+      "src");
   gst_element_add_pad (GST_ELEMENT (statistics), statistics->srcpad);
 
   statistics->timer = NULL;
