@@ -4,7 +4,17 @@
 #include <stdlib.h>
 #include "mem.h"
 
-static GstElement*
+GstStaticPadTemplate srctemplate = GST_STATIC_PAD_TEMPLATE ("src",
+    GST_PAD_SRC,
+    GST_PAD_ALWAYS,
+    GST_STATIC_CAPS_ANY);
+
+GstStaticPadTemplate sinktemplate = GST_STATIC_PAD_TEMPLATE ("sink",
+    GST_PAD_SINK,
+    GST_PAD_ALWAYS,
+    GST_STATIC_CAPS_ANY);
+
+static GstElement *
 create_thread (void)
 {
   GstElement *thread;
@@ -21,7 +31,7 @@ create_thread (void)
   return thread;
 }
 
-static GstElement*
+static GstElement *
 create_thread_ghostpads (void)
 {
   GstElement *thread;
@@ -30,14 +40,19 @@ create_thread_ghostpads (void)
   thread = gst_thread_new ("testthread");
   element1 = gst_element_new ();
   gst_element_set_name (element1, "test1");
-  gst_element_add_pad (element1, gst_pad_new ("src1", GST_PAD_SRC));
+  gst_element_add_pad (element1,
+      gst_pad_new_from_template (gst_static_pad_template_get (&srctemplate),
+          "src1"));
   gst_bin_add (GST_BIN (thread), element1);
   element2 = gst_element_new ();
   gst_element_set_name (element2, "test2");
-  gst_element_add_pad (element2, gst_pad_new ("sink1", GST_PAD_SINK));
+  gst_element_add_pad (element1,
+      gst_pad_new_from_template (gst_static_pad_template_get (&sinktemplate),
+          "sink1"));
   gst_bin_add (GST_BIN (thread), element2);
   gst_element_link (element1, "src1", element2, "sink1");
-  gst_element_add_ghost_pad (thread, gst_element_get_pad (element2, "sink1"), "sink1");
+  gst_element_add_ghost_pad (thread, gst_element_get_pad (element2, "sink1"),
+      "sink1");
 
   return thread;
 }
@@ -129,7 +144,7 @@ add_remove_test4 (void)
 }
 
 int
-main (int argc, gchar *argv[])
+main (int argc, gchar * argv[])
 {
   GstElement *thread, *element;
   long usage1;
@@ -143,17 +158,17 @@ main (int argc, gchar *argv[])
     iters = ITERS;
 
   g_print ("starting test\n");
-  usage1 = vmsize();
+  usage1 = vmsize ();
 
   thread = gst_thread_new ("somethread");
   gst_object_unref (GST_OBJECT (thread));
-  g_print ("create/unref new thread %ld\n", vmsize()-usage1);
+  g_print ("create/unref new thread %ld\n", vmsize () - usage1);
 
-  for (i=0; i<iters;i++) {
+  for (i = 0; i < iters; i++) {
     thread = gst_thread_new ("somethread");
     gst_object_unref (GST_OBJECT (thread));
   }
-  g_print ("create/unref %d threads %ld\n", iters, vmsize()-usage1);
+  g_print ("create/unref %d threads %ld\n", iters, vmsize () - usage1);
 
   thread = gst_thread_new ("somethread");
   g_assert (GST_OBJECT_FLOATING (thread));
@@ -161,62 +176,64 @@ main (int argc, gchar *argv[])
   gst_object_sink (GST_OBJECT (thread));
   g_assert (!GST_OBJECT_FLOATING (thread));
   gst_object_unref (GST_OBJECT (thread));
-  g_print ("create/ref/sink/unref new thread %ld\n", vmsize()-usage1);
+  g_print ("create/ref/sink/unref new thread %ld\n", vmsize () - usage1);
 
 
-  for (i=0; i<iters;i++) {
+  for (i = 0; i < iters; i++) {
     thread = gst_thread_new ("somethread");
     gst_object_ref (GST_OBJECT (thread));
     gst_object_sink (GST_OBJECT (thread));
     gst_object_unref (GST_OBJECT (thread));
   }
-  g_print ("create/ref/sink/unref %d threads %ld\n", iters, vmsize()-usage1);
+  g_print ("create/ref/sink/unref %d threads %ld\n", iters, vmsize () - usage1);
 
   thread = gst_thread_new ("somethread");
   g_assert (!GST_OBJECT_DESTROYED (thread));
   gst_object_unref (GST_OBJECT (thread));
   g_assert (GST_OBJECT_DESTROYED (thread));
   gst_object_unref (GST_OBJECT (thread));
-  g_print ("create/destroy/unref new thread %ld\n", vmsize()-usage1);
-  
-  for (i=0; i<iters;i++) {
+  g_print ("create/destroy/unref new thread %ld\n", vmsize () - usage1);
+
+  for (i = 0; i < iters; i++) {
     thread = gst_thread_new ("somethread");
     gst_object_unref (GST_OBJECT (thread));
     gst_object_unref (GST_OBJECT (thread));
   }
-  g_print ("create/destroy/unref %d thread %ld\n", iters, vmsize()-usage1);
+  g_print ("create/destroy/unref %d thread %ld\n", iters, vmsize () - usage1);
 
   thread = gst_thread_new ("somethread");
   gst_object_ref (GST_OBJECT (thread));
   gst_object_unref (GST_OBJECT (thread));
   gst_object_unref (GST_OBJECT (thread));
-  g_print ("create/ref/unref/unref new thread %ld\n", vmsize()-usage1);
-  
-  for (i=0; i<iters;i++) {
+  g_print ("create/ref/unref/unref new thread %ld\n", vmsize () - usage1);
+
+  for (i = 0; i < iters; i++) {
     thread = gst_thread_new ("somethread");
     gst_object_ref (GST_OBJECT (thread));
     gst_object_unref (GST_OBJECT (thread));
     gst_object_unref (GST_OBJECT (thread));
   }
-  g_print ("create/ref/unref/unref %d thread %ld\n", iters, vmsize()-usage1);
+  g_print ("create/ref/unref/unref %d thread %ld\n", iters, vmsize () - usage1);
 
   thread = gst_thread_new ("somethread");
   gst_object_ref (GST_OBJECT (thread));
   gst_object_unref (GST_OBJECT (thread));
   gst_object_unref (GST_OBJECT (thread));
   gst_object_unref (GST_OBJECT (thread));
-  g_print ("craete/ref/destroy/unref/unref new thread %ld\n", vmsize()-usage1);
-  
-  for (i=0; i<iters;i++) {
+  g_print ("craete/ref/destroy/unref/unref new thread %ld\n",
+      vmsize () - usage1);
+
+  for (i = 0; i < iters; i++) {
     thread = gst_thread_new ("somethread");
     gst_object_ref (GST_OBJECT (thread));
     gst_object_unref (GST_OBJECT (thread));
     gst_object_unref (GST_OBJECT (thread));
     gst_object_unref (GST_OBJECT (thread));
   }
-  g_print ("craete/ref/destroy/unref/unref %d threads %ld\n", iters, vmsize()-usage1);
+  g_print ("craete/ref/destroy/unref/unref %d threads %ld\n", iters,
+      vmsize () - usage1);
 
-  for (i=0; i<iters;i++) {
+  for (i = 0; i < iters; i++) {
     thread = gst_thread_new ("somethread");
     gst_object_ref (GST_OBJECT (thread));
     gst_element_set_name (thread, "testing123");
@@ -225,57 +242,63 @@ main (int argc, gchar *argv[])
     gst_object_unref (GST_OBJECT (thread));
     gst_object_unref (GST_OBJECT (thread));
   }
-  g_print ("craete/ref/destroy/unref/unref %d threads with name %ld\n", iters, vmsize()-usage1);
+  g_print ("craete/ref/destroy/unref/unref %d threads with name %ld\n", iters,
+      vmsize () - usage1);
 
   thread = gst_thread_new ("somethread");
-  for (i=0; i<iters;i++) {
+  for (i = 0; i < iters; i++) {
     gst_element_set_name (thread, "testing");
   }
   gst_object_unref (GST_OBJECT (thread));
-  g_print ("set name %d times %ld\n", iters, vmsize()-usage1);
+  g_print ("set name %d times %ld\n", iters, vmsize () - usage1);
 
-  for (i=0; i<iters;i++) {
+  for (i = 0; i < iters; i++) {
     thread = gst_thread_new ("somethread");
     element = gst_element_new ();
     gst_element_set_name (element, "test1");
     gst_bin_add (GST_BIN (thread), element);
     gst_object_unref (GST_OBJECT (thread));
   }
-  g_print ("create/unref %d thread with one element %ld\n", iters, vmsize()-usage1);
+  g_print ("create/unref %d thread with one element %ld\n", iters,
+      vmsize () - usage1);
 
-  for (i=0; i<iters;i++) {
-    thread = create_thread();
+  for (i = 0; i < iters; i++) {
+    thread = create_thread ();
     gst_object_unref (GST_OBJECT (thread));
   }
-  g_print ("create/unref %d thread with children %ld\n", iters, vmsize()-usage1);
+  g_print ("create/unref %d thread with children %ld\n", iters,
+      vmsize () - usage1);
 
-  for (i=0; i<iters/2;i++) {
-    thread = create_thread_ghostpads();
+  for (i = 0; i < iters / 2; i++) {
+    thread = create_thread_ghostpads ();
     gst_object_unref (GST_OBJECT (thread));
   }
-  g_print ("create/unref %d thread with children and ghostpads %ld\n", iters/2, vmsize()-usage1);
+  g_print ("create/unref %d thread with children and ghostpads %ld\n",
+      iters / 2, vmsize () - usage1);
 
-  for (i=0; i<iters;i++) {
-    add_remove_test1();
+  for (i = 0; i < iters; i++) {
+    add_remove_test1 ();
   }
-  g_print ("add/remove test1 %d in thread %ld\n", iters, vmsize()-usage1);
+  g_print ("add/remove test1 %d in thread %ld\n", iters, vmsize () - usage1);
 
-  for (i=0; i<iters;i++) {
-    add_remove_test2();
+  for (i = 0; i < iters; i++) {
+    add_remove_test2 ();
   }
-  g_print ("add/remove test2 %d in thread %ld\n", iters, vmsize()-usage1);
+  g_print ("add/remove test2 %d in thread %ld\n", iters, vmsize () - usage1);
 
-  for (i=0; i<iters;i++) {
-    add_remove_test3();
+  for (i = 0; i < iters; i++) {
+    add_remove_test3 ();
   }
-  g_print ("add/destroy/remove test3 %d in thread %ld\n", iters, vmsize()-usage1);
+  g_print ("add/destroy/remove test3 %d in thread %ld\n", iters,
+      vmsize () - usage1);
 
-  for (i=0; i<iters;i++) {
-    add_remove_test4();
+  for (i = 0; i < iters; i++) {
+    add_remove_test4 ();
   }
-  g_print ("add/destroy/remove test4 %d in thread %ld\n", iters, vmsize()-usage1);
+  g_print ("add/destroy/remove test4 %d in thread %ld\n", iters,
+      vmsize () - usage1);
 
-  g_print ("leaked: %ld\n", vmsize()-usage1);
+  g_print ("leaked: %ld\n", vmsize () - usage1);
 
-  return (vmsize()-usage1 ? -1 : 0);
+  return (vmsize () - usage1 ? -1 : 0);
 }
